/*
 * flawctl.c
 *
 * Copyright Stephen C. Tweedie 1997
 *
 * Simple utility to set and clear bad block ranges on a flawd badblock
 * debugging ramdisk.
 */

#include <stdlib.h>
#include <stdio.h>
#include <unistd.h>
#include <sys/fcntl.h>

#include "flawd.h"

char *progname;

void useage(int exit_code)
{
	fprintf (stderr,
		 "Useage:\n"
		 "%s /dev/flawd? [-[cs] start_block [+count]] ...\n",
		 progname);
	exit (exit_code);
}

int main (int argc, char *argv[])
{
	enum {SET, CLEAR} mode;
	int start, count;
	char *devname;
	int fd, rc;
	__FLAWD_SECTOR_RANGE flawd_args;
	
	progname = *argv;
	
	argv++, argc--;
	
	if (!argc)
		useage (1);
	devname = *argv;
	argv++, argc--;
	
	fd = open(devname, O_RDONLY, 0);
	if (fd < 0) {
		perror ("open");
		exit (1);
	}
	
	while (argc > 0) {
		if (!strcmp(*argv, "-h"))
			useage (0);
		else if (!strcmp(*argv, "-s"))
			mode = SET;
		else if (!strcmp(*argv, "-c"))
			mode = CLEAR;
		else 
			useage (1);

		argv++, argc--;
		if (!argc)
			useage (1);
		
		do {
			start = strtol (*argv, 0, 0);
			argv++, argc--;
			
			if (argc && *argv[0] != '-') {
				count = strtol (*argv, 0, 0);
				argv++, argc--;
			} else
				count = 1;
			
			printf ("Mode %d, start %d, count %d.\n",
				mode, start, count);
			
			flawd_args.start = start;
			flawd_args.count = count;
			rc = ioctl (fd, 
				    (mode == SET)
				    ? FLAWD_SET_BAD_SECTORS
				    : FLAWD_CLEAR_BAD_SECTORS,
				    &flawd_args);
			if (rc < 0) {
				perror ("ioctl");
				exit (1);
			}
		} while (argc && *argv[0] != '-');
	}
	close (fd);
}

